@@ -1,4 +1,4 @@
-from pyg_base import is_date, ulist, logger, is_primitive, is_str, is_strs, as_list, get_cache, Dict, dictable, list_instances
+from pyg_base import eq, is_date, ulist, is_primitive, is_str, is_strs, as_list, get_cache, Dict, dictable, list_instances
 from pyg_encoders import cell_root, root_path, pd_read_parquet, pickle_load, pd_read_csv, dictable_decode
 from pyg_npy import pd_read_npy
 from pyg_cell._types import _get_mode, _get_qq, DBS, QQ, CLSS
@@ -327,7 +327,7 @@ class db_cell(cell):
         db = db()
         missing = ulist(db._pk) - self.keys()
         if len(missing):
-            logger.warning('WARN: document not saved as some keys are missing %s'%missing)
+            self._logger.warning('WARN: document not saved as some keys are missing %s'%missing)
             return self            
         ref = type(doc)(**cell_clear(dict(doc)))
         try:
@@ -407,7 +407,7 @@ class db_cell(cell):
         pk = ulist(db._pk)
         missing = pk - self.keys()
         if len(missing):
-            logger.warning('WARN: document not loaded as some keys are missing %s'%missing)
+            self._logger.warning('WARN: document not loaded as some keys are missing %s'%missing)
             return self            
         address = self._address
         kwargs = {k : self[k] for k in pk}
@@ -442,9 +442,8 @@ class db_cell(cell):
                 excluded_keys += as_list(mode)
             elif is_date(mode):
                 excluded_keys += [_id]
-            updated_inputs = [k for k, v in self._inputs.items() if k in saved and v is not None and 
-                              (type(saved[k])!=type(v) or (is_primitive(v) and saved[k]!=v))]
             update = (saved / None) - excluded_keys
+            updated_inputs = [k for k, v in self._inputs.items() if k in saved and v is not None and not isinstance(v, cell) and not eq(saved[k], v)]
             self.update(update)
             if len(updated_inputs):
                 self[_updated] = None
@@ -675,20 +674,21 @@ def _get_cell(table = None, db = None, url = None, schema = None, server = None,
                 t = DBS[mode](db = db, table = table, server = server or url, schema = schema, pk = pk, doc = doc or _doc)
                 qq = t.table.c
         else:
-            return GRAPH[address]
+            return GRAPH[address].copy()
         address = t.address + kwargs_address
         if _from_memory and deleted in (None, False): # we want the standard cell
             if address not in GRAPH:
                 doc = _load_asof(t, kwargs, deleted, qq)
                 if isinstance(doc, cell):
                     GRAPH[doc._address] = doc
-                return doc
+                return doc.copy()
             else:
-                return GRAPH[address]
+                return GRAPH[address].copy()
         else:
             return _load_asof(t, kwargs, deleted, qq) # must not overwrite live version. User wants a specific deleted version
     else:
-        return GRAPH[address]
+        print('got here')
+        return GRAPH[address].copy()
 
 
 def load_cell(table = None, db = None, url = None, schema = None, server = None, deleted = None, doc = None, **kwargs):
